@@ -1,3 +1,5 @@
+#![allow(non_snake_case)]
+
 /// Errors from which parser cannot recover.
 #[derive(Debug)]
 #[non_exhaustive]
@@ -17,7 +19,7 @@ impl From<std::io::Error> for FatalError {
 /// which it occurs. For example, unrecognized attribute will simply be skipped
 /// without affecting anything around it, while unrecognized element will have
 /// all of its contents skipped.
-#[derive(Debug)]
+#[derive(Debug, PartialEq, Eq)]
 #[non_exhaustive]
 pub enum Error {
     UnexpectedElement {
@@ -45,6 +47,11 @@ pub enum Error {
         xpath: String,
         desc: String,
     },
+    ParseIntError {
+        xpath: String,
+        text: String,
+        error: std::num::ParseIntError,
+    },
     Internal {
         desc: &'static str,
     },
@@ -92,6 +99,8 @@ pub enum RegistryChild {
     /// Container for all published Vulkan specification extensions.
     Extensions(Extensions),
 
+    Formats(Formats),
+
     SpirvExtensions(SpirvExtensions),
 
     SpirvCapabilities(SpirvCapabilities),
@@ -374,6 +383,12 @@ pub struct TypeMemberDefinition {
     )]
     pub limittype: Option<String>,
 
+    #[cfg_attr(
+        feature = "serialize",
+        serde(default, skip_serializing_if = "is_default")
+    )]
+    pub objecttype: Option<String>,
+
     #[cfg_attr(
         feature = "serialize",
         serde(default, skip_serializing_if = "is_default")
@@ -744,6 +759,12 @@ pub struct CommandParam {
     )]
     pub noautovalidity: Option<String>,
 
+    #[cfg_attr(
+        feature = "serialize",
+        serde(default, skip_serializing_if = "is_default")
+    )]
+    pub objecttype: Option<String>,
+
     /// The definition of this parameter.
     #[cfg_attr(
         feature = "serialize",
@@ -1022,6 +1043,72 @@ pub enum InterfaceItem {
     },
 }
 
+pub type Formats = CommentedChildren<Format>;
+
+#[derive(Debug, Clone, PartialEq, Eq, Default)]
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+#[non_exhaustive]
+pub struct Format {
+    pub name: String,
+    pub class: String,
+    pub blockSize: u8,
+    pub texelsPerBlock: u8,
+
+    #[cfg_attr(
+        feature = "serialize",
+        serde(default, skip_serializing_if = "is_default")
+    )]
+    pub blockExtent: Option<String>,
+
+    #[cfg_attr(
+        feature = "serialize",
+        serde(default, skip_serializing_if = "is_default")
+    )]
+    pub packed: Option<u8>,
+
+    #[cfg_attr(
+        feature = "serialize",
+        serde(default, skip_serializing_if = "is_default")
+    )]
+    pub compressed: Option<String>,
+
+    #[cfg_attr(
+        feature = "serialize",
+        serde(default, skip_serializing_if = "is_default")
+    )]
+    pub chroma: Option<String>,
+
+    pub children: Vec<FormatChild>,
+}
+
+#[derive(Debug, Clone, PartialEq, Eq)]
+#[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+#[non_exhaustive]
+pub enum FormatChild {
+    #[non_exhaustive]
+    Component {
+        name: String,
+        bits: String,
+        numericFormat: String,
+        #[cfg_attr(
+            feature = "serialize",
+            serde(default, skip_serializing_if = "is_default")
+        )]
+        planeIndex: Option<u8>,
+    },
+
+    #[non_exhaustive]
+    Plane {
+        index: u8,
+        widthDivisor: u8,
+        heightDivisor: u8,
+        compatible: String,
+    },
+
+    #[non_exhaustive]
+    SpirvImageFormat { name: String },
+}
+
 #[derive(Debug, Clone, PartialEq, Eq, Default)]
 #[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
 #[non_exhaustive]
@@ -1041,6 +1128,7 @@ pub struct NameWithType {
 
 #[derive(Debug, Clone, PartialEq, Eq, Default)]
 #[cfg_attr(feature = "serialize", derive(Serialize, Deserialize))]
+#[non_exhaustive]
 pub struct CommentedChildren<T> {
     #[cfg_attr(
         feature = "serialize",
